package bcyrill;

import reversi.*;
import java.util.Vector;

/**
 * Abstrakte Player Klasse
 * 
 * @author bcyrill
 * 
 */
abstract public class Player {

	/**
	 * Die Farbe des Spielers.
	 */
	protected int color = 0;

	/**
	 * Das Timeout des Spiels (nicht relevant im Blatt 7).
	 */
	protected long timeout = 0;

	/**
	 * Konstante, die vom Benutzer eigegeben werden kann, um zu passen.
	 */
	protected final static String PASSEN = "p";

	/**
	 * Testet ob ein Zug gueltig ist auf Grund des benachbarten Steines
	 * 
	 * @param zug
	 *            zu ueberpruefender Zug
	 * @param gb
	 *            zu pruefende Situation
	 * @param drow
	 *            Nachbarstein Row
	 * @param dcol
	 *            Nachbarstein Col
	 * @return <code>true</code> falls der Zug gueltig ist sonst
	 *         <code>false</code>
	 */
	final protected boolean checkValid(Coordinates zug, GameBoard gb, int drow,
			int dcol) {
		int player2 = 3 - this.color;
		// Koordinaten des Nachbarsteins
		int n_row = zug.getRow() + drow;
		int n_col = zug.getCol() + dcol;
		// Reversi Spielregeln (x Spieler 1, o Spieler 2)
		try {
			// (xo)
			if (gb.getPosition(new Coordinates(n_row, n_col)) != player2) {
				return false;
			}
			do {
				// (xo..oo)
				n_row += drow;
				n_col += dcol;
			} while (gb.getPosition(new Coordinates(n_row, n_col)) == player2);

			// (auf xo..oo folgt x)
			return (gb.getPosition(new Coordinates(n_row, n_col)) == color);
		} catch (OutOfBoundsException e) {
			return false;
		}
	}

	/**
	 * Testet ob ein Zug gueltig ist.
	 * 
	 * @param zug
	 *            zu ueberpruefender Zug
	 * @param gb
	 *            zu pruefende Situation
	 * @return <code>true</code> falls der Zug gueltig ist sonst
	 *         <code>false</code>
	 */
	final protected boolean checkMove(Coordinates zug, GameBoard gb) {
		try {
			if (gb.getPosition(zug) == GameBoard.EMPTY) {
				// versuche alle Richtungen
				return (checkValid(zug, gb, -1, 0)
						|| checkValid(zug, gb, -1, -1)
						|| checkValid(zug, gb, 0, -1)
						|| checkValid(zug, gb, 1, -1)
						|| checkValid(zug, gb, 1, 0)
						|| checkValid(zug, gb, 1, 1)
						|| checkValid(zug, gb, 0, 1) || checkValid(zug, gb, -1,
						1));
			} else {
				return false;
			}
		} catch (OutOfBoundsException e) {
			return false;
		}
	}// end checkMove()

	/**
	 * Bruteforce: Teste alle Felder auf moeglichen Zug
	 * 
	 * @param gb
	 *            zu testende Situation
	 * @return Vector mit allen moeglichen / gueltigen Zuegen
	 */
	final protected Vector movePossible(GameBoard gb) {
		Vector m = new Vector();
		// Teste alle Felder auf gueltigen Zug
		for (int row = 1; row <= gb.getSize(); row++) {
			for (int col = 1; col <= gb.getSize(); col++) {
				Coordinates coord = new Coordinates(row, col);
				try {
					if (gb.getPosition(coord) != GameBoard.EMPTY) {
						continue;
					}
				} catch (OutOfBoundsException e) {
					return null;
				}
				skip: for (int a = -1; a <= 1; a++) {
					for (int b = -1; b <= 1; b++) {
						if (a == 0 && b == 0) {
							// unbenoetigter Fall
							continue;
						}
						if (this.checkValid(coord, gb, a, b)) {
							m.add(coord);
							// Feld wurde gezaehlt muss nicht weiter ueberprueft
							// werden, -> fortfahren mit dem naechsten Feld
							break skip;
						}
					}
				}
			}
		}
		return m;
	}

}
